@@ -3,9 +3,8 @@ use wasm_bindgen::prelude::*;
 use wasm_bindgen::JsCast;
 use crate::window::same_origin;
 use web_sys::{AudioBuffer, AudioBufferSourceNode, AudioNode, MediaElementAudioSourceNode, HtmlAudioElement, AudioContext};
-use std::rc::Rc;
-use std::cell::RefCell;
-use std::cell::Cell;
+use std::sync::atomic::{AtomicU64, Ordering};
+use std::{rc::Rc, cell::RefCell};
 
 pub enum AudioClip {
     Regular(AudioClipState),
@@ -32,14 +31,15 @@ where
 
 pub struct AudioClipState {
     destination: AudioNode,
-    start_offset: Cell<f64>,
-    start_time: Cell<f64>,
     ctx: AudioContext,
     is_loop: bool,
     source_node: AudioSourceNode, 
-    on_ended: Option<OnEndedCallback>
+    on_ended: Option<OnEndedCallback>,
+    _start_offset: AtomicU64,
+    _start_time: AtomicU64, 
 }
 
+
 type OnEndedCallback = Closure<dyn FnMut() -> ()>;
 
 enum AudioSourceNode {
@@ -48,15 +48,20 @@ enum AudioSourceNode {
     //AudioWorkletNode for streaming?
 }
 
+impl Drop for AudioClip {
+    fn drop(&mut self) {
+        log::info!("clip dropped!");
+    }
+}
 impl AudioClip {
-    pub fn new<F>(ctx: &AudioContext, source: AudioSource, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error>  
+    pub fn new<F, A: Into<AudioSource>>(ctx: &AudioContext, source: A, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error>  
     where
         F: FnMut() -> () + 'static,
     {
         Ok(Self::Regular(AudioClipState::new(ctx, source, destination, options)?))
     }
 
-    pub fn new_oneshot<F>(ctx: &AudioContext, source: AudioSource, destionation: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error>  
+    pub fn new_oneshot<F, A: Into<AudioSource>>(ctx: &AudioContext, source: A, destionation: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error>  
     where
         F: FnMut() -> () + 'static,
     {
@@ -106,12 +111,12 @@ impl AudioClip {
 
 impl AudioClipState {
 
-    pub fn new<F>(ctx: &AudioContext, source: AudioSource, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error> 
+    pub fn new<F, A: Into<AudioSource>>(ctx: &AudioContext, source: A, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Self, Error> 
     where
         F: FnMut() -> () + 'static,
     {
 
-        let _self = match source {
+        let _self = match source.into() {
             AudioSource::Url(url) => {
                 //uses an element internally
                 //maybe in the future use streaming api?
@@ -143,8 +148,8 @@ impl AudioClipState {
 
                 Self { 
                     destination,
-                    start_offset: Cell::new(0.0),
-                    start_time: Cell::new(if options.auto_play { ctx.current_time() } else { 0.0 }),
+                    _start_offset: AtomicU64::new(0),
+                    _start_time: AtomicU64::new(if options.auto_play { ctx.current_time().to_bits() } else { 0 }),
                     ctx: ctx.clone(),
                     is_loop: options.is_loop,
                     on_ended,
@@ -157,8 +162,9 @@ impl AudioClipState {
 
                 let _self = Self { 
                     destination,
-                    start_offset: Cell::new(0.0),
-                    start_time: Cell::new(0.0),
+                    _start_offset: AtomicU64::new(0),
+                    //need this too? _start_time: AtomicU64::new(if options.auto_play { ctx.current_time().to_bits() } else { 0 }),
+                    _start_time: AtomicU64::new(0),
                     ctx: ctx.clone(),
                     is_loop: options.is_loop,
                     on_ended,
@@ -181,11 +187,31 @@ impl AudioClipState {
         Ok(_self)
     }
 
+    fn get_start_offset(&self) -> f64 {
+        let value:u64 = self._start_offset.load(Ordering::SeqCst);
+        f64::from_bits(value)
+    }
+
+    fn get_start_time(&self) -> f64 {
+        let value:u64 = self._start_time.load(Ordering::SeqCst);
+        f64::from_bits(value)
+    }
+
+    fn set_start_offset(&self, value: f64) {
+        let value = value.to_bits();
+        self._start_offset.store(value, Ordering::SeqCst);
+    }
+
+    fn set_start_time(&self, value: f64) {
+        let value = value.to_bits();
+        self._start_time.store(value, Ordering::SeqCst);
+    }
+
     //A regular audio clip is effectively a one-shot since dropping will stop it
     //But it can be annoying to need to keep it around in memory until playing is finished
     //So this one-shot will drop itself when finished
     //(the state is imperatively dropped via the clip's Drop impl too)
-    pub fn new_oneshot<F>(ctx: &AudioContext, source: AudioSource, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Rc<RefCell<Option<Self>>>, Error>
+    pub fn new_oneshot<F, A: Into<AudioSource>>(ctx: &AudioContext, source: A, destination: AudioNode, options: AudioClipOptions<F>) -> Result<Rc<RefCell<Option<Self>>>, Error>
     where
         F: FnMut() -> () + 'static,
     {
@@ -218,9 +244,9 @@ impl AudioClipState {
 
     // https://books.google.co.il/books?id=eSPyRuL8b7UC&pg=PA14&lpg=PA14&dq=WebAudioApi+pause+source&source=bl&ots=ZezP65-Qtk&sig=ACfU3U1Kx9N2vBtXbxt1ysq6RLTkfuwvBA&hl=en&sa=X&ved=2ahUKEwj5oYyf9qDxAhWH-aQKHQ8tBGQQ6AEwCXoECBcQAw#v=onepage&q=WebAudioApi%20pause%20source&f=false
     pub fn pause(&self) -> Result<(), Error> {
-        let diff = self.ctx.current_time() - self.start_time.get();
-        let start_offset = self.start_offset.get() + diff;
-        self.start_offset.replace(start_offset);
+        let diff = self.ctx.current_time() - self.get_start_time();
+        let start_offset = self.get_start_offset() + diff;
+        self.set_start_offset(start_offset);
 
         match &self.source_node {
             AudioSourceNode::Buffer(buffer, node_ref) => {
@@ -239,8 +265,8 @@ impl AudioClipState {
     }
 
     pub fn play(&self) -> Result<(), Error> {
-        self.start_time.replace(self.ctx.current_time());
-        let start_offset = self.start_offset.get();
+        self.set_start_time(self.ctx.current_time());
+        let start_offset = self.get_start_offset();
 
         match &self.source_node {
             AudioSourceNode::Buffer(buffer, node_ref) => {
